######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 12/03/2022
### license = PSF
### version = 3.2.0
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os
import subprocess

#Local Packages
import next.read_config
import next.tools

def build():
    """Build project from current directory
    """
    
    try:
        
        # Current directory
        this_dir = os.getcwd()
        
        try:
            
            # Read config of proyect
            config_obj = read_config.read_config(this_dir)

            # If the configuration is not empty
            if config_obj != False:

                try:
                    # Try create build_dir
                    os.mkdir(config_obj.get("build_dir"))
                    
                    # Message(Successful): The build_dir directory was created
                    tools.message_successful(this_dir + '/' + config_obj.get("build_dir"))
                    
                except:
                    # Message(Waiting): The build_dir folder already exists
                    tools.message_warning("Warning " + this_dir + "/" + config_obj.get("build_dir") +  " folder already exists")
                
                # Entering the directory build_dir
                os.chdir(config_obj.get("build_dir"))
                
                # Message(Waiting): Build Proyect
                tools.message_waiting("Build Proyect")

                # Command to build the project of Cmake
                subprocess.run([
                    "cmake", this_dir + "/.", 
                    "-G" + config_obj.get("build_system"), 
                    "-DCMAKE_CXX_COMPILER=" +  config_obj.get("cxx_compiler"), 
                    "-DCMAKE_C_COMPILER=" +  config_obj.get("c_compiler"),
                    "-D" + config_obj.get("type_project") +"=on" ] + config_obj.get("cmake_flags"))

                # Command to build the project of build_system
                subprocess.run([config_obj.get("build_system_exe")] + config_obj.get("build_system_flags"))
            # The configuration is empty
            else:
                # Message(Warning): The configuration is empty
                tools.message_warning("The configuration is empty")

        except OSError as exc:
            
            # Message(Error): OSError generate
            tools.message_error(str(exc))
    except OSError as err:
        
        # Message(Error): OSError generate
        tools.message_error(str(err))
        
        # Exit to program
        exit()


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 28/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os
import shutil

#Local Packages
import next.tools

def create(name, build_dir, name_build, build_system_exe, c_compiler, cxx_compiler, build_system, type_project):

    # Get current Directory
    this_dir = os.getcwd()

    next_dir = ""
    try:
        #Search NEXT_PACKAGES_DIR
        next_dir = os.environ['NEXT_DIR']
        
        # Message(Info): NEXT_DIR find in 
        tools.message_info('NEXT_DIR in: ' + next_dir)

    except:
        # Message(Error): Not Find NEXT_DIR
        tools.message_error('It was not found ENV NEXT_DIR')  
        exit()

    # Message(Waiting): Create a proyect of next in
    tools.message_waiting('Create a proyect of next in: ' + this_dir + '/' + name)
    try:
        # Default Type Proyect
        base_project = "empty_executable/"
        if(type_project):
            
            # If type is a Library
            if(type_project == "static_library" or type_project == "dynamic_library"):
                base_project = "empty_library/"

        # Create the dir
        os.mkdir(name)

        # Get nextEmptyProjectDir
        next_empty_project_dir = next_dir + "/assets/" + base_project

        # Copy the nextEmptyProjectDir in new project {name}
        shutil.copytree(next_empty_project_dir, name, dirs_exist_ok=True)

        # View the dir of new project {name}
        dir_new_project = os.getcwd() + "/" + name

        tools.remplace_in_file(dir_new_project + "/config.yaml", "__name_project__", name)
        tools.remplace_in_file(dir_new_project + "/CMakeLists.txt", "__name_project__", name)

        if build_dir:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__build_dir__", build_dir)
        else:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__build_dir__", "build")

        if name_build:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__name_build__", name_build)
            tools.remplace_in_file(dir_new_project + "/CMakeLists.txt", "__name_build__", name_build)
        else:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__name_build__", "app")
            tools.remplace_in_file(dir_new_project + "/CMakeLists.txt", "__name_build__", "app")

        if build_system_exe and build_system:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__build_system_exe__", build_system_exe)
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__build_system__", build_system)
        else:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__build_system_exe__", "ninja")
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__build_system__", "Ninja")

        if c_compiler:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__c_compiler__", c_compiler)
        else:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__c_compiler__", "gcc")

        if cxx_compiler:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__cxx_compiler__", cxx_compiler)
        else:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__cxx_compiler__", "g++")

        if type_project:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__type_project__", type_project)

            if(type_project == "static_library"):
                setup_library(dir_new_project, name, name_build, type_project)
            elif(type_project == "dynamic_library"):
                setup_library(dir_new_project, name, name_build)
                
        else:
            tools.remplace_in_file(dir_new_project + "/config.yaml", "__type_project__", "executable")
            
        # Message(Successful): Create a proyect of next in
        tools.message_successful('Create a proyect of next in: ' + this_dir + '/' + name)

    except OSError as exc:
        
        # Message(Error): OSError generate
        tools.message_error(str(exc))

def setup_library(dir_new_project, name, name_build, type_project):
    tools.remplace_in_file(dir_new_project + "/export/export_library.cmake", "__LIB_NAME__", name.upper())

    if name_build:
        if(type_project == "static_library"):
            tools.remplace_in_file(dir_new_project + "/export/export_library.cmake", "__LIB_BINARY__", name_build + ".a")
        elif(type_project == "dynamic_library"):
            tools.remplace_in_file(dir_new_project + "/export/export_library.cmake", "__LIB_BINARY__", name_build + ".so")
    else:
        if(type_project == "static_library"):
            tools.remplace_in_file(dir_new_project + "/export/export_library.cmake", "__LIB_BINARY__", "app_static" + ".a")
        elif(type_project == "dynamic_library"):
            tools.remplace_in_file(dir_new_project + "/export/export_library.cmake", "__LIB_BINARY__", "app_dynamic" + ".so")



######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 20/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os

#Local Packages
import next.tools

def check_env():
    """Check if it exists NEXT_PACKAGES_DIR
    """
    try:
        #Search NEXT_PACKAGES_DIR
        next_dir = os.environ['NEXT_PACKAGES_DIR']
        
        # Message(Info): Next_PACKAGES_DIR in
        tools.message_info('NEXT_PACKAGES_DIR in: ' + next_dir)
    except:
        # Message(Warning): Not Find NEXT_PACKAGES_DIR
        tools.message_warning("It was not found ENV NEXT_PACKAGES_DIR")

        #Want to create NEXT_PACKAGES_DIR
        tools.message_info("Create NEXT_PACKAGES_DIR y/n: ")
        create_next_packages_dir_flag = input()

        #Create or Not NEXT_PACKAGES_DIR
        if create_next_packages_dir_flag == "y":

            #Select dir for NEXT_PACKAGES_DIR
            select_next_packages_dir()
        else:

            # Message(Warning): Is not created NEXT_PACKAGES_DIR
            tools.message_warning("Warning!!! you must create NEXT_PACKAGES_DIR")
        exit()

def select_next_packages_dir():
    """Select dir for NEXT_PACKAGES_DIR
    """

    #Search home dir
    home_dir = os.environ['HOME']

    #Default dor for NEXT_PACKAGES_DIR
    next_packages_dir = home_dir + "/NextPackages"

    # Message(Info): Acept dir for NEXT_PACKAGES_DIR
    tools.message_info("NEXT_PACKAGES_DIR will be created in: " + home_dir + " y/n:")
    next_packages_dir_flag = input()

    #Use default dor for NEXT_PACKAGES_DIR or not
    if next_packages_dir_flag == "y":

        #Create dir for NEXT_PACKAGES_DIR
        create_next_packages_dir(next_packages_dir)
    else:
        #Message(Info): Select dir for NEXT_PACKAGES_DIR
        tools.message_info("Enter the route where you want NEXT_PACKAGES_DIR: ")
        next_packages_dir = input()

        #Create dir for NEXT_PACKAGES_DIR
        create_next_packages_dir(next_packages_dir + "/NextPackages")

def create_next_packages_dir(dir):
    """Create dir for NEXT_PACKAGES_DIR

    Args:
        dir (str): Direction of Next Packages
    """

    #verify that it does not exist dir for NEXT_PACKAGES_DIR
    if not os.path.isdir(dir):
        
        #Create dir for NEXT_PACKAGES_DIR
        os.mkdir(dir)

    # Message(Successful): Create NEXT_PACKAGES_DIR
    tools.message_successful('Now add environment variable NEXT_PACKAGES_DIR='+ dir)



######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 29/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os
import subprocess

#Local Packages
import next.read_config
import next.tools

def exce(command):
    """Execute a command
    """
    
    # Get current Directory
    this_dir = os.getcwd()
    try:
        
        # Read config of proyect
        config_obj = read_config.read_config(this_dir)

        # If the configuration is not empty
        if config_obj != False:
            
            try:
                #Get Commands
                commands = config_obj.get('commands')

                # Get keys
                keys_commands = commands.keys()
                
                # Value of Command 
                value_command = ''
                
                # Flag
                find = False
                
                # Search command
                for x in keys_commands:
                    
                    # Find Command
                    if x == command :
                        
                        # Set Flag
                        find = True
                        
                        # Save value of command
                        value_command = commands[command]
                        
                        # Break the Search
                        break
                
                # If Find the Command
                if( find ):
                    
                    # Message(Waiting): Executing the command
                    tools.message_waiting('Executing the command: ' + command)
                    subprocess.run(list(value_command.split(" ")))
                else: 
                    # Message(Error): Executing the command
                    tools.message_error('Command not found: ' + command)
                
            except OSError as err:
                # Message(Error): OSError generate
                tools.message_error(str(err))
    except OSError as exc:
        
        # Message(Error): OSError generate
        tools.message_error(str(exc))
        
        # Exit to program
        exit()


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 28/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os

#Local Packages
import next.tools    

def _this_is_a_dir(dir):
    """Identify if this is Dir

    Args:
        dir (str): Direction of proyect

    Returns:
        bool: Flag
    """
    
    # Flag 
    done = False

    # Is a Directory
    if os.path.isdir(dir):
        done = True
        
        # Message(Successful): Is a directory
        tools.message_successful(dir + " Is a directory")
    else:
        
        # Message(Error): Not ss a directory
        tools.message_error(dir + " Not is a directory")
    
    return done

def write_property(config_obj, dir):
    """Write a new Property

    Args:
        config_obj (Config_t): Configuration of project
        dir (str): Directory

    Returns:
        flag: Flag successful
    """
    
    # Identify if this is a directory
    if _this_is_a_dir(dir):

        # Get Map of Config_object
        config_map = config_obj.to_map()
                
        # Write Config
        with open('config.yaml', 'w') as file:
            documents = config_obj.yaml.dump(config_map, file)
        
        # Remplace chars '[' and ']'
        tools.remplace_in_file("config.yaml", "'[", "[")
        tools.remplace_in_file("config.yaml", "]'", "]")
        
        return True

    return False


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 12/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import shutil
import os

#Local Packages
import next.read_config
import next.tools

def clean():
    """Clean Build of current Project
    """
    
    # Get current Directory
    this_dir = os.getcwd()
    try:
        
        # Read config of proyect
        config_obj = read_config.read_config(this_dir)

        # If the configuration is not empty
        if config_obj != False:
            
            try:
                # Remove the directory "build_dir"
                shutil.rmtree(config_obj.get("build_dir"))
                
                # Message(Successful): The build_dir delete
                tools.message_successful('Clean ' + this_dir + '/' + config_obj.get("build_dir"))
            except OSError as err:
                # Message(Error): OSError generate
                tools.message_error(str(err))
    except OSError as exc:
        
        # Message(Error): OSError generate
        tools.message_error(str(exc))
        
        # Exit to program
        exit()


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 20/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os

#Local Packages
import next.read_config
import next.tools

def get(property):
    """Get a property to the current project

    Args:
        property (str): name of property

    Returns:
        value_of_property([str, null]): value of property
    """

    # default value of property
    value_of_property = "null"

    # alone Next version
    if(property != ""):

        # View the dir of current project {name}
        dir_project = os.getcwd()

        #Read config of current project
        config_obj = read_config.read_config(dir_project)

        #Wrapper for properties
        value_of_property = config_obj.get(property)
        
        #If it was added correctly
        if(value_of_property != "null"):
            
            # Message(Successful): Getting property
            tools.message_successful('Getting property ' + property + ': ' + value_of_property)
            
        else:
            # Message(Error): Could not add
            tools.message_error('Could not get ' + property)

    #Value of new property ([str, null])
    return value_of_property


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 28/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os

#Local Packages
import next.config_t
import next.tools

def _this_is_a_dir(dir):
    """Identify if this is Dir

    Args:
        dir (str): Direction of proyect

    Returns:
        bool: Flag
    """
    
    # Flag 
    done = False

    # Is a Directory
    if os.path.isdir(dir):
        done = True
        
        # Message(Successful): Is a directory
        tools.message_successful(dir + " Is a directory")
    else:
        
        # Message(Error): Not ss a directory
        tools.message_error(dir + " Not is a directory")
    
    return done

def _exists_config_file(dir):
    """Identify if it exists config.yaml

    Args:
        dir (str): Direction of project

    Returns:
        bool: Flag
    """
    
    # Flag
    done = False
    try:
        
        #Try open config.yaml
        config_file = open( dir + "/config.yaml", "r")
        config_file.close()
        
        # Message(Successful): Is a directory
        tools.message_successful("Exists config.yaml in :" + dir)
        
        done = True
    except:
        # Message(Error): Not is a directory
        tools.message_error("Not exists config.yaml in :" + dir)
    
    return done

def read_config(dir):
    """Read Config of Project

    Args:
        dir (str): Dir of Project

    Returns:
        [Config_t, bool]: Data or Flag
    """
    
    # Identify if this is a Proyect of Next
    if _this_is_a_dir(dir) and _exists_config_file(dir):
        
        # Create Config_t object
        config_obj = config_t.Config_t(dir)
        
        # Message(Successful): Is a directory
        tools.message_successful(dir + " Is a project of Next")
                
        # Return Config_t object
        return config_obj
    
    # Message(Error): Not is a project Next
    tools.message_error(dir + " Not is a project of Next")
    return False


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 12/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import shutil
import os
import datetime
import platform

#Local Packages
import next.read_config
import next.tools

# String for add in cmake/vendor.cmake
vendor_basic = """
#__NAME_LIB_LOWER__ added __DATE__
include(__FILE_IMPORT_LIB__)

set( INCLUDE_LIBS
    ${INCLUDE_LIBS}
    ${__NAME_LIB_UPPER___INCLUDE_DIR}
)

set( LIBS
    ${LIBS}
    ${__NAME_LIB_UPPER__})
"""

def use_path(library_dir):
    """Add library to this project from full path

    Args:
        library_dir (str): full path of library
    """
    
    # Get current Directory
    this_dir = os.getcwd()
    
    #Get NEXT_DIR env
    next_dir = ""
    try:
        #Search NEXT_PACKAGES_DIR
        next_dir = os.environ['NEXT_DIR']
        
        # Message(Info): NEXT_DIR find in 
        tools.message_info('NEXT_DIR in: ' + next_dir)

    except:
        # Message(Error): Not Find NEXT_DIR
        tools.message_error('It was not found ENV NEXT_DIR')  
        exit()

    try:
        
        # Read config of proyect
        config_obj = read_config.read_config(this_dir)
        
        # Read config of library
        config_lib = read_config.read_config(library_dir)

        # If the configuration is not empty
        if config_obj != False and config_lib != False:
            
            try:
                
                # Get NEXT_DIR/assets/import_base.cmake
                import_base_file = next_dir + "/assets/import_base.cmake"
                
                #Get properties of library
                name_project_lib = config_lib.get('name_project')
                
                name_build_lib = config_lib.get('name_build')
                
                build_dir_lib = config_lib.get('build_dir')
                
                # Generate name of import cmake
                file_import_lib = 'cmake/' + name_project_lib + '.cmake'
                
                # Copy the import_base.cmake
                shutil.copyfile(import_base_file,file_import_lib)
                
            #### Verify exist binary
        
                # Try Generate Default name_build_lib_abs
                name_build_lib_abs = library_dir + '/' + build_dir_lib + '/' + name_build_lib
                
                if os.path.isfile(name_build_lib_abs):
                    
                    # Message(Info): Library binary file found
                    tools.message_info('Library binary file found: ' + name_build_lib_abs)
                else:
                    
                    # Message(Warning): Library binary file not found
                    tools.message_warning('Library binary not found')
                    
                    # Message(Info): Search file
                    tools.message_info('Want to search yes/no')
                    
                    res = input()
                    
                #### Search likely binaries
                    if res == 'yes':
                        # Message(Waiting): Search file
                        tools.message_waiting('Search')
                        # Verify that it exists name_build_lib
                        
                        likely_binaries_local = []
                        
                        likely_binaries_abs = []
                        
                        files_build_abs = tools.absoluteFilePaths(library_dir + '/' + build_dir_lib)
                        
                        files_build_local = os.listdir(library_dir + '/' + build_dir_lib)
                        
                        # Get likely files relative path
                        for file in files_build_local:
                            if file.find(name_build_lib) != -1:
                                likely_binaries_local.append(file)
                                
                        # Get likely files full path
                        for file_local in likely_binaries_local:
                            for file_abs in files_build_abs:
                                if file_abs.find(file_local) != -1:
                                    likely_binaries_abs.append(file_abs)
                        
                        # Message(Info): Mathches Found
                        tools.message_info('Matches found')
                        i = 1
                        for f in likely_binaries_abs:
                            print( str(i) + ') ' + f)
                            i = i + 1
                        
                        # Message(Info): Select the binary
                        tools.message_info('Select a Binary \'n\' to cancelr')
                        res = input()
                        
                        # Binary not selected
                        if res == 'n':
                            # Message(Warning): Select the binary
                            tools.message_warning('You will need to add the library binary manually.')
                            
                            name_build_lib_abs = ''
                        else:
                            # Adding the binary
                            name_build_lib_abs = likely_binaries_abs[int(res) - 1]
                        
                    else:
                        # Message(Warning): Select the binary
                        tools.message_warning('You will need to add the library binary manually.')
                        
                        # Clean the name_build_lib_abs
                        name_build_lib_abs = ''
                        
                # Configuration of file_import_lib
                tools.remplace_in_file(file_import_lib, '__NAME_PROJECT_UPPER_CASE__', name_project_lib.upper())
                
                tools.remplace_in_file(file_import_lib, '__DIR_LIB__', library_dir)
                
                tools.remplace_in_file(file_import_lib, '__FILE_BUILD_ABS__', name_build_lib_abs)

                # Remplace \ route for /
                system = platform.system()
                if system != 'Linux':
                    tools.remplace_in_file(file_import_lib, '\\', '/')
                
                
                # Write in cmake/vendor.cmake
                file_vendor = open(this_dir + '/cmake/vendor.cmake', 'a')
                
                file_vendor.write(vendor_basic)
                
                file_vendor.close()
                
                # Configuration of cmake/vendor.cmake
                tools.remplace_in_file(this_dir + '/cmake/vendor.cmake', '__NAME_LIB_UPPER__', name_project_lib.upper())
                
                tools.remplace_in_file(this_dir + '/cmake/vendor.cmake', '__NAME_LIB_LOWER__', name_project_lib.lower())

                tools.remplace_in_file(this_dir + '/cmake/vendor.cmake', '__FILE_IMPORT_LIB__', file_import_lib)
                
                tools.remplace_in_file(this_dir + '/cmake/vendor.cmake', '__DATE__', str(datetime.datetime.now()))     
                
                # Message(Successful): Library added
                tools.message_successful(library_dir + 'added in: ' + this_dir)
                
            except OSError as err:
                # Message(Error): OSError generate
                tools.message_error(str(err))
    except OSError as exc:
        
        # Message(Error): OSError generate
        tools.message_error(str(exc))
        
        # Exit to program
        exit()


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 28/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

# Update 28/03/2022
_VERSION = "3.2.0"

#System Packages
import os

#Local Packages
import next.read_config
import next.tools


def version():
    # Message(Info): Next Version
    tools.message_info("Next version: " + _VERSION)

def version_all():
    
    # Next Packages Dierctory
    next_packages_dir = ""
    try:
        #Search NEXT_PACKAGES_DIR
        next_packages_dir = os.environ['NEXT_PACKAGES_DIR']
        
        # Message(Info): NEXT_PACKAGES_DIR in:
        tools.message_info("NEXT_PACKAGES_DIR in: " +  next_packages_dir)

    except:
        # Message(Error): Not Find NEXT_PACKAGES_DIR
        tools.message_error("It was not found ENV NEXT_PACKAGES_DIR in func --version_next.version_all()--")
        exit()

    # Get Subdirectories 
    list_next_packages = os.listdir(next_packages_dir)

    for next_pakage_dir in list_next_packages:

        # Read COnfig for DIrectory
        config_obj = read_config.read_config(next_packages_dir+ "/" + next_pakage_dir)

        # If exists Config_t
        if config_obj != False:
            
            # Message(Info): Get Version of Proects
            tools.message_info(config_obj.name_project + " " + config_obj.version + "\n")
    


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 20/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

# Packages Dependencies
import ruamel.yaml

# Packges Local
import next.tools

def listToString(l):
    """Convert a list to a string

    Args:
        l (list): List to convert

    Returns:
        str: string
    """
    
    # initialize an empty string
    str1 = " " 
    
    # return string  
    return (str1.join(l))

class Config_t:

    _data = {}
    file = ''
    yaml = ''

    def __init__(self, dir):
        """Initialize a Config_t

        Args:
            dir (str): Direction of project
        """
        # Create a Yaml
        self.yaml = ruamel.yaml.YAML()
        self.yaml.preserve_quotes = True

        #Read file
        self.file = open( dir + "/config.yaml", "r")

        #Write Data
        self._data = self.yaml.load(self.file)

    def print(self):
        """Print the Config_t
        """
        print( "name_project: "         + self._data["name_project"])
        print( "description: "          + self._data["description"])
        print( "version: "              + self._data["version"])
        print( "build_dir: "            + self._data["build_dir"])
        print( "name_build: "           + self._data["name_build"])
        print( "build_system_exe: "     + self._data["build_system_exe"])
        print( "c_compiler: "           + self._data["c_compiler"])
        print( "cxx_compiler: "         + self._data["cxx_compiler"])
        print( "build_system: "         + self._data["build_system"])
        print( "cmake_flags: "          + listToString(self._data["cmake_flags"]))
        print( "build_system_flags: "   + listToString(self._data["build_system_flags"]))

    def get(self, property):
        """Get the property

        Args:
            property (_type_): Name of Property

        Returns:
            str: Value of property
        """
        try:
            value = self._data[property]
        except:
            value = "null"
        return value

    def set(self, property, value):
        """Set the property

        Args:
            property (str): Name of property
            value (obj_yaml): Value of property

        Returns:
            obj_yaml: Value of property
        """
        try:
            current_value = self._data[property]
            self._data[property] = value
            new_value = self._data[property]
        except:
            tools.message_error("Property does not exist")
            new_value = "null"
        return new_value

    def add(self, property, value):
        """Set the property

        Args:
            property (str): Name of property
            value (obj_yaml): Value of property

        Returns:
            obj_yaml: Value of property
        """
        try:
            self._data[property] = value
            new_value = self._data[property]
        except:
            tools.message_error("Property does not exist")
            new_value = "null"
        return new_value

    def to_map(self):
        """Convert Config_t to Map

        Returns:
            Map: Map of Data
        """
        return self._data


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 12/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

# Packages Systems
import os
import platform

# Packages Dependencies
import ruamel.yaml

def remplace_in_file(file_location, old_text, new_text):
    try:
        #input file
        fin = open(file_location, "rt")

        old_data = fin.readlines()

        new_data = ""

        #for each line in the input file
        for line in old_data:
            #read replace the string and write to output file
            new_data += line.replace(old_text, new_text)
        #close input and output files
        fin.close()

        #output file to write the result to
        fout = open(file_location, "wt")

        #Write new data
        fout.write(new_data)

        #Close output file
        fout.close()
    except BaseException as err:
        print("Not remplace " + old_text + " in " + file_location)
        print(f"Unexpected {err=}, {type(err)=}")
        
def load_env():
    """load the environment variables in the current execution
    """
    try:
        env_yaml = ''
        
        file = ''
        
        _data = {}
        
        #Get Home Dir
        system = platform.system()
        
        if system == 'Linux':
            print('Linux System')
            home_dir = os.environ['HOME']
        else:
            print('Windows System')
            home_dir = os.environ['LOCALAPPDATA']
        
        next_env_file_dir = home_dir + '/.next/env.yaml'
        
        env_yaml = ruamel.yaml.YAML()
        
        env_yaml.preserve_quotes = True

        #Read the file
        file = open( next_env_file_dir, "r")

        #Safe the data
        _data = env_yaml.load(file)
        
        #Load the environment variables in the current execution
        for x in _data:
            print(x + '= ' + _data[x])
            os.environ[x] = _data[x]
        
    except:
        
        print("Error at Load Env")
        
from colorama  import Fore
from colorama import Style
from colorama import Back
from colorama import init

# For colorama in Windows load ANSI
system = platform.system()
if system != 'Linux':
    init(convert = True)

def message_error(str):
    print(f'{Fore.RED}{Style.BRIGHT} <<ERROR>> {Style.RESET_ALL}' + str)
    
def message_warning(str):
    print(f'{Fore.YELLOW}{Style.BRIGHT} <<WARNING>> {Style.RESET_ALL}' + str)
    
def message_successful(str):
    print(f'{Fore.LIGHTGREEN_EX}{Style.BRIGHT} <<SUCCESSFUL>> {Style.RESET_ALL}' + str)
    
def message_info(str):
    print(f'{Fore.WHITE}{Style.BRIGHT} <<INFO>> {Style.RESET_ALL}' + str)
    
def message_waiting(str):
    print(f'{Fore.BLUE}{Style.BRIGHT} <<WAITING...>> {Style.RESET_ALL}' + str)
    
def absoluteFilePaths(directory):
    for dirpath,_,filenames in os.walk(directory):
        for f in filenames:
            yield os.path.abspath(os.path.join(dirpath, f))


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 20/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Development
######################################################################

#System Packages
import os

#Local Packages
import next.read_config
import next.write_config
import next.tools

def add(property, value):
    """Adds a new property to the current project

    Args:
        property (str): name of new property
        value (str): value of new property

    Returns:
        value_of_property([str, null]): value of new property
    """

    # default value of property
    value_of_property = "null"

    # alone Next version
    if(property != ""):

        # View the dir of current project {name}
        dir_project = os.getcwd()

        #Read config of current project
        config_obj = read_config.read_config(dir_project)

        #Wrapper for properties
        value_of_property = config_obj.add(property, value)

        #If it was added correctly
        if(value_of_property != "null"):
            
            #Write the new config
            write_config.write_property(config_obj, dir_project)
            
            # Message(Successful): Added property
            tools.message_successful('Added property ' + property + ': ' + value)
            
        else:
            # Message(Error): Could not add
            tools.message_error('Could not add ' + property + ': ' + value)

    #Value of new property ([str, null])
    return value_of_property


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 29/01/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

# Packages Dependencies
import click

#Local Packages
import next.info_next
import next.version_next
import next.create_next
import next.build_next
import next.run_next
import next.clean_next
import next.config_env
import next.config_get
import next.config_set
import next.config_add
import next.tools
import next.exce_next
import next.use_next

### Update 29/03/2022
### ✓ create                   Create a new Next project.
### ✓ build                    Build this project
### ✓ run                      Run your app
### ✓ clean                    Remove the binaries
### ✓ version                  List Next and plugins version.
### ✓ info                     Print Info verbose of Next
### ✓ add                      Add to property of current Next Project
### ✓ get                      Get to property of current Next Project
### ✓ set                      Set to property of current Next Project
### ✓ exce                     Excecute a command perzonalized

### × use                      Use a new library
### × remove                   Remove a library
### × install                  Install a Plugin
### × upgrade                  Upgrade a Plugin or Next
### × doctor                   Show information about the installed tooling.

@click.group()
def main():
    tools.load_env()
    pass

@main.command('info', short_help='view info the Next')
def info():
    info_next.info()

@main.command('version', short_help='view version the Next')
@click.option('--all',default=0, required=False, help='view version of all NextPackages installed <default=0>')
def version(all):
    # alone Next version
    if(all == 0):
        version_next.version()
    # all NextPackages
    elif(all == 1):
        version_next.version_all()
    # Error Not show any version
    else:
        exit()

@main.command('check_env', short_help='check env the NextPackages')
def check_env():
    config_env.check_env()

@main.command('create', short_help='Create a new project of Next', options_metavar='<name> <options>')
@click.argument('name', required=True, type=str, metavar='')
@click.option('--build_dir', required=False, type=str, help='Select Build Dir')
@click.option('--name_build', required=False, type=str, help='Select name of build')
@click.option('--build_system_exe', required=False, type=str, help='Select Build System executable')
@click.option('--c_compiler', required=False, type=str, help='Select C Compiler')
@click.option('--cxx_compiler', required=False, type=str, help='Select C++ Compiler')
@click.option('--build_system', required=False, type=str, help='Select Build System')
@click.option('--type_project', required=False, type=str, help='Select Type Project')
def create(name, build_dir, name_build, build_system_exe, c_compiler, cxx_compiler, build_system, type_project):
    create_next.create(name, build_dir, name_build, build_system_exe, c_compiler, cxx_compiler, build_system, type_project)

@main.command('build', short_help='Build a project of Next')
def build():
    build_next.build()

@main.command('run', short_help='Run a project of Next')
def run():
    run_next.run()

@main.command('clean', short_help='Clean a project of Next')
def clean():
    clean_next.clean()

@main.command('get', short_help='Get property of current Next Project')
@click.option('--property',default="name", required=True, help='Select property of current Next Project <default=name>')
def get(property):
    value_of_property = config_get.get(property)
    if isinstance(value_of_property, list):
        value_in_str = ''
        value_in_str += '['
        for x in value_of_property:
            value_in_str += x + ','
        len_value = len(value_in_str)
        value_in_str = value_in_str[:len_value-1]
        value_in_str += ']' 
    else:
        value_in_str = value_of_property
    
    print(property + ": " + value_in_str)

@main.command('set', short_help='Set property of current Next Project')
@click.option('--property',default="name", required=True, help='Select property of current Next Project <default=name>')
@click.option('--value',default="name", required=True, help='Select value of current Next Project <default=null>')
def set(property, value):
    config_set.set(property, value)


@main.command('add', short_help='Add to property of current Next Project')
@click.option('--property',default="name", required=True, help='Select property of current Next Project <default=name>')
@click.option('--value',default="name", required=True, help='Select value of current Next Project <default=null>')
def add(property, value):
    config_add.add(property, value)
    
@main.command('exce', short_help='Add to property of current Next Project')
@click.argument('command')
def exce(command):
    exce_next.exce(command)
    
@main.command('use', short_help='Add new library in current project')
@click.argument('library')
def use(library):
    use_next.use_path(library)

#if __name__ == "__main__":
main()



######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 12/03/2022
### license = PSF
### version = 3.2.0
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################


from setuptools import setup, find_packages

setup(
    name = 'next-pm',
    packages = ['next'],   
    include_package_data=True,    # muy importante para que se incluyan archivos sin extension .py
    version = '3.2.1',
    description = 'Next es un administrador de proyectos de C/C++, es diseñado como una solucion a la administracion que requieren este tipo de proyectos.)',
    author='Rafael Zamora',
    author_email="rafa.zamora.ram@gmail.com",
    license="PSF",
    url="https://github.com/reitmas32/Next/tree/v3.2.0.pipy",
    classifiers = ["Programming Language :: Python :: 3",\
        "License :: OSI Approved :: Python Software Foundation License",\
        "Development Status :: 5 - Production/Stable", "Intended Audience :: Developers", \
        "Operating System :: OS Independent", \
        "Topic :: Software Development :: Build Tools"],
    keywords=['C/C++', 'package', 'libraries', 'developer', 'manager',
              'dependency', 'tool', 'c', 'c++', 'cpp'],
    entry_points={
        'console_scripts': [
            'next=next.next:main'
        ],
    },
    )


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 20/03/2022
### license = PSF
### version = 3.2.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

#System Packages
import os
import subprocess

#Local Packages
import next.read_config
import next.tools

def run():
    """Run executable of project
    """

    # Get current directory
    this_dir = os.getcwd()
    
    try:
        
        #Read Config of Project
        config_obj = read_config.read_config(this_dir)

        # If this is a project of Next
        if config_obj != False:
            
            # Enter the directory bulid_dir
            os.chdir(config_obj.get("build_dir"))
            
            # Message(Waiting): Run executable
            tools.message_waiting('Run executable')
            
            # Run executable
            subprocess.run(["./" + config_obj.get("name_build")])
    except OSError as exc:
        
        # Message(Error): OSError generate
        tools.message_error(str(exc))
        
        # Exit to program
        exit()


######################################################################
### author = Rafael Zamora 
### copyright = Copyright 2020-2022, Next Project 
### date = 07/01/2022
### license = PSF
### version = 3.0.0 
### maintainer = Rafael Zamora 
### email = rafa.zamora.ram@gmail.com 
### status = Production
######################################################################

# Next info
_info_string = """
Manage your C/C++ Proyect development.
Common commands:
  next create <output directory>
    Create a new C++ project in the specified directory.
  next run [options]
    Run your C/C++ Proyect application.
Usage: next <command> [arguments]
Global options:
--help                  Print this usage information.
info                    Print Info verbose of Next
Available commands:
  create                   Create a new Next project.
  build                    Build this project
  run                      Run your app
  clean                    Remove the binaries
  doctor                   Show information about the installed tooling.
  version                  List Next and plugins version.
  install                  Install a Plugin
  upgrade                  Upgrade a Plugin or Next
"""

def info():
    print(_info_string)